#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "usart.h"

#define DURATA_SECVENTA 100
#define NUM_NOTE 2
#define ALARM_INIT 'I'
#define ALARM_STOP 'S'
#define ALARM_CLOSE 'C'

double frecventa_alarma[NUM_NOTE] = {2096.00, 2793.83};
char Data_in;
bool alarm_status = false;
bool alarm_stop = false;
bool alarm_init = false;
bool send_message = true;
bool first_message = true;

/* Initializare ADC */
void ADC_init(void)
{
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (5 << ADPS0);
}

/* Conversia valorii de la ADC. */
int ADC_get(uint8_t channel)
{
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while (ADCSRA & (1 << ADSC))
		;

	return (int)ADC;
}

/* Genereaza o nota folosind buzzer-ul. */
void alarm_sequence(int nota)
{
	float perioada_nota = 1000000 / frecventa_alarma[nota];
	int i, j;

	if (nota == 0)
	{
		// Aprind led-ul 1
		PORTB |= (1 << PB0);
		for (i = 0; i < DURATA_SECVENTA; i++)
		{
			// Pornesc buzzer-ul
			PORTD ^= (1 << PD5);

			for (j = 0; j < perioada_nota / 2; ++j)
				_delay_us(2);
		}
		// Opresc Led-ul 1
		PORTB &= ~(1 << PB0);
	}
	else if (nota == 1)
	{
		// Aprind led-ul 2
		PORTD |= (1 << PD6); //Turns ON All LEDs
		// _delay_ms(400);		  //1 second delay
		for (i = 0; i < DURATA_SECVENTA; i++)
		{
			// Pornesc buzzer-ul
			PORTD ^= (1 << PD5);

			for (j = 0; j < perioada_nota / 2; ++j)
				_delay_us(2);
		}
		// Opresc Led-ul 2
		PORTD &= ~(1 << PD6); //Turns OFF All LEDs
	}
}

void alarm()
{
	int i;
	for (i = 0; i < 2; i++)
	{
		alarm_sequence(i);
	}
}

void stop_alarm()
{
	alarm_status = false;
	send_message = true;
	PORTB &= ~(1 << PB0);
	PORTD &= ~(1 << PD6);
	PORTC |= (1 << PD5);
}

void alarm_starter()
{
	Data_in = USART0_receive();
	// USART0_clear();
	switch (Data_in)
	{
	case ALARM_INIT:
		alarm_init = true;
		break;
	case ALARM_STOP:
		stop_alarm();
		break;
	case ALARM_CLOSE:
		stop_alarm();
		alarm_init = false;
		first_message = true;
		break;
	default:
		break;
	}
}

/*
 * Genereaza note folosind buzzer-ul. La detectarea unei intreruperi 
 * a razei, se va aprinde si ledul legat la PD7.
 */
void alarm_module(void)
{

	/* Setez portul B ca iesire */
	DDRB = 0xff;
	/* Initializez USART cu 9600 baud rate */
	USART0_init();
	/* Initializare ADC. */
	ADC_init();

	/* Setez pininul PB2*/
	DDRB &= ~(1 << PB2);

	/* Activez rezistenta de pull-up pentru PB2*/
	PORTB |= (1 << PB2);

	/* Setez pinul buzzerului si ledurile ca iesire. */
	DDRD |= (1 << PD5) | (1 << PB0) | (1 << PD6);
	;

	/* Opresc buzzer-ul. */
	PORTD ^= (1 << PD5);

	while (1)
	{
		/* 
		 * Daca s-a intrerupt un laser, va porni semnalul acustic al alarmei
		 * Se aprinde ledul
		 * Se timite mesajul de "Alerta" prin bluethooth 
		 * La primirea caracterului 'S' alarma se va opri
		 * Sau la apasarea butonului legat la PB2
		 */
		alarm_starter();
		
		if (first_message && alarm_init) {
			USART0_print("Alarm is online\n");
			first_message = false;
		}	
		int val = ADC_get(1);
		if (alarm_init)
		{
			if (val < 400)
			{
				if (send_message)
				{
					USART0_print("Alert\n");
					send_message = false;
				}
				if (!alarm_status)
					alarm_status = true;
			}

			if (alarm_status)
			{
				alarm();
			}

			if ((PINB & (1 << PB2)) == 0)
			{
				stop_alarm();
			}
		}
	}
}

int main(void)
{
	alarm_module();

	while (1)
		;

	return 0;
}